//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Clipbrd.hpp>
#include "App_Document.h"
#include "Moa_Files.h"
#include "Moa_ShellLinks.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_document {
//---------------------------------------------------------------------------
AnsiString __fastcall TDocument::EnsureValidPath(const AnsiString& APath)
{
	if(APath.Length() == 0) {
		return APath;
	}
	AnsiString LPath = APath;
	try {
		TPathUtilities::AssureAbsolute(LPath);
		LPath = TFileUtilities::GetLongPath(LPath);
		LPath = ExpandFileName(LPath);
	}
	catch(EFileError& LException) {
		throw EAppError(AnsiString().sprintf(
			"The program cannot accept the path \"%s\".",
			APath.c_str()));
	}
	return LPath;
}
//---------------------------------------------------------------------------
void __fastcall TDocument::SetFile(const AnsiString& AFile)
{
	AnsiString LFile = EnsureValidPath(AFile);
	if(LFile == File) {
		return;
	}
	FFile = LFile;
	DoFileChange();
}
//---------------------------------------------------------------------------
void __fastcall TDocument::SetDereference(bool ADereference)
{
	if((ADereference ? 1 : 0) == (Dereference ? 1 : 0)) {
		return;
	}
	FDereference = ADereference;
	DoDereferenceChange();
}
//---------------------------------------------------------------------------
void __fastcall TDocument::SetAutoClose(bool AAutoClose)
{
	if((AAutoClose ? 1 : 0) == (AutoClose ? 1 : 0)) {
		return;
	}
	FAutoClose = AAutoClose;
	DoAutoCloseChange();
}
//---------------------------------------------------------------------------
void __fastcall TDocument::DoFileChange()
{
	if(OnFileChange != NULL) {
		OnFileChange(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocument::DoDereferenceChange()
{
	if(OnDereferenceChange != NULL) {
		OnDereferenceChange(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocument::DoAutoCloseChange()
{
	if(OnAutoCloseChange != NULL) {
		OnAutoCloseChange(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TDocument::TDocument() :
	TModel(),
	FFile(),
	FDereference(true),
	FAutoClose(true),
	FOnFileChange(NULL),
	FOnDereferenceChange(NULL),
	FOnAutoCloseChange(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDocument::~TDocument()
{
	// Do nothing
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocument::DetermineFile()
{
	AnsiString LFile = File;
	if(!IsEmpty() && Dereference) {
		LFile = TShortcutFileUtilities::EnsureDereferenced(LFile);
	}
	return LFile;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocument::DetermineTitle()
{
	if(IsEmpty()) {
		return "(None)";
	}
	else {
		return ExtractFileName(File);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocument::DetermineIcon(TIcon* AIcon)
{
	if(IsEmpty()) {
		AIcon->Handle = NULL;
	}
	else {
		SHFILEINFO LFileInfo = {0};
		if(::SHGetFileInfo(
			File.c_str(),
			0,
			&LFileInfo,
			sizeof(SHFILEINFO),
			SHGFI_ICON | SHGFI_SHELLICONSIZE | SHGFI_SMALLICON) == NULL) {
			throw EAppError("The program could not get the icon.");
		}
		AIcon->Handle = LFileInfo.hIcon;
	}
}
//---------------------------------------------------------------------------
bool __fastcall TDocument::IsEmpty()
{
	return File.Length() == 0;
}
//---------------------------------------------------------------------------
bool __fastcall TDocument::IsShortcut()
{
	if(IsEmpty()) {
		throw EAppError("The program cannot determine the file is a shortcut.");
	}
	return TShortcutFileUtilities::IsShortcutFile(File);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentCommands::DetermineCommandPrompt()
{
	OSVERSIONINFO LVersionInfo = {0};
	LVersionInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if(!::GetVersionEx(&LVersionInfo)) {
		throw EAppError("The program cannot get the version of the current operating system.");
	}
	switch(LVersionInfo.dwPlatformId) {
		case VER_PLATFORM_WIN32s:
		case VER_PLATFORM_WIN32_WINDOWS:
			return "COMMAND.COM";
		case VER_PLATFORM_WIN32_NT:
		default:
			return "cmd.exe";
	};
}
//---------------------------------------------------------------------------
__fastcall TDocumentCommands::TDocumentCommands(TDocument* ADocument) :
	TCommands(),
	FDocument(ADocument)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDocumentCommands::~TDocumentCommands()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Open()
{
	if(!CanOpen()) {
		HandleFeatureUnavailable();
	}
	AnsiString LFile = Document->DetermineFile();
	if(FileExists(LFile)) {
		LFile = ExtractFileDir(LFile);
	}
	AnsiString LDirectory = LFile;
	SHELLEXECUTEINFO LExecuteInfo = {0};
	LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI;
	LExecuteInfo.lpVerb = "open";
	LExecuteInfo.lpFile = LFile.c_str();
	LExecuteInfo.lpDirectory = LDirectory.c_str();
	LExecuteInfo.nShow = SW_SHOWNORMAL;
	if(!::ShellExecuteEx(&LExecuteInfo)) {
		throw EAppError(AnsiString().sprintf(
			"The program could not open the directory \"%s\".",
			LFile.c_str()));
	}
	if(Document->AutoClose) {
		Application->MainForm->Close();
	}
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanOpen()
{
	return !Document->IsEmpty();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Explore()
{
	if(!CanExplore()) {
		HandleFeatureUnavailable();
	}
	AnsiString LFile = Document->DetermineFile();
	if(FileExists(LFile)) {
		LFile = ExtractFileDir(LFile);
	}
	AnsiString LDirectory = LFile;
	SHELLEXECUTEINFO LExecuteInfo = {0};
	LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI;
	LExecuteInfo.lpVerb = "explore";
	LExecuteInfo.lpFile = LFile.c_str();
	LExecuteInfo.lpDirectory = LDirectory.c_str();
	LExecuteInfo.nShow = SW_SHOWNORMAL;
	if(!::ShellExecuteEx(&LExecuteInfo)) {
		throw EAppError(AnsiString().sprintf(
			"The program could not explore the directory \"%s\".",
			LFile.c_str()));
	}
	if(Document->AutoClose) {
		Application->MainForm->Close();
	}
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanExplore()
{
	return !Document->IsEmpty();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::CopyPath()
{
	if(!CanCopyPath()) {
		HandleFeatureUnavailable();
	}
	Clipboard()->AsText = Document->DetermineFile();
	if(Document->AutoClose) {
		Application->MainForm->Close();
	}
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanCopyPath()
{
	return !Document->IsEmpty();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::StartCommandPrompt()
{
	if(!CanStartCommandPrompt()) {
		HandleFeatureUnavailable();
	}
	AnsiString LFile = DetermineCommandPrompt();
	AnsiString LDirectory = Document->DetermineFile();
	if(FileExists(LDirectory)) {
		LDirectory = ExtractFileDir(LDirectory);
	}
	SHELLEXECUTEINFO LExecuteInfo = {0};
	LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI;
	LExecuteInfo.lpVerb = "open";
	LExecuteInfo.lpFile = LFile.c_str();                              
	LExecuteInfo.lpDirectory = LDirectory.c_str();
	LExecuteInfo.nShow = SW_SHOWNORMAL;
	if(!::ShellExecuteEx(&LExecuteInfo)) {
		throw EAppError("The program could not start the commad prompt.");
	}
	if(Document->AutoClose) {
		Application->MainForm->Close();
	}
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanStartCommandPrompt()
{
	return !Document->IsEmpty();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::ShowProperties()
{
	if(!CanShowProperties()) {
		HandleFeatureUnavailable();
	}
	AnsiString LFile = Document->File;
	SHELLEXECUTEINFO LExecuteInfo = {0};
	LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI | SEE_MASK_INVOKEIDLIST;
	LExecuteInfo.lpVerb = "properties";
	LExecuteInfo.lpFile = LFile.c_str();
	LExecuteInfo.nShow = SW_SHOWNORMAL;
	if(!::ShellExecuteEx(&LExecuteInfo)) {
		throw EAppError(AnsiString().sprintf(
			"The program could not show the properties of the file \"%s\".",
			LFile.c_str()));
	}
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanShowProperties()
{
	return !Document->IsEmpty();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::DropFiles(const TStringDynArray& AFiles)
{
	if(!CanDropFiles()) {
		HandleFeatureUnavailable();
	}
	if(AFiles.Length == 0) {
		return;
	}
	for(int LI = 0; LI < AFiles.Length; ++LI) {
		if(Document->IsEmpty()) {
			Document->File = AFiles[LI];
		}
		else {
			AnsiString LFile = Application->ExeName;
			AnsiString LParameters = AnsiQuotedStr(AFiles[LI], '\"');
			SHELLEXECUTEINFO LExecuteInfo = {0};
			LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
			LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI;
			LExecuteInfo.lpVerb = "open";
			LExecuteInfo.lpFile = LFile.c_str();
			LExecuteInfo.lpParameters = LParameters.c_str();
			LExecuteInfo.nShow = SW_SHOWNORMAL;
			if(!::ShellExecuteEx(&LExecuteInfo)) {
				throw EAppError("The program could not start the program.");
			}
		}
	}
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanDropFiles()
{
	return true;
}
//---------------------------------------------------------------------------
} // End namespace App_document
//---------------------------------------------------------------------------

